unit MainUnit;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, ComCtrls;

type
  TMainForm = class(TForm)
    VoltScroll: TScrollBox;
    ControlPanel: TPanel;
    CalCombo: TComboBox;
    VoltSpacerBevel: TBevel;
    ADPollTime: TTimer;
    GoButton: TButton;
    StopButton: TButton;
    RangeScroll: TScrollBox;
    RangeSpacerBevel: TBevel;
    CalCheck: TCheckBox;
    TestLabel: TLabel;
    procedure FormCreate(Sender: TObject);
    procedure RangeComboChange(Sender: TObject);
    procedure ADPollTimeTimer(Sender: TObject);
    procedure GoButtonClick(Sender: TObject);
    procedure StopButtonClick(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  MainForm: TMainForm;

implementation

{$R *.DFM}

uses
  Math,
  AIOUSB,
  TheseUtils, Temperature, TypeK, LM335,
  DetectUnit;

const
  NAN: Double = 0/0;
  
type
  TUSBAI16128Config = packed record
    ChannelRange: array[$0..$F] of Byte;
    CalibMode: Byte;
    TrigMode: Byte;
    StartStopCh: Byte;
    Oversample: Byte;
    StartStopChEx: Byte; //MUX start and end channel extension register.
  end;

var
  DeviceIndex: LongWord;
  Config: TUSBAI16128Config;
  ConfigBufSize: LongWord;
  RangeCombo: array[$0..$F] of TComboBox;
  VoltLabel: array of TLabel;
  MAXChannels: Integer;
  Counts: array of Word;

procedure TMainForm.FormCreate(Sender: TObject);
var
  Status, PID: LongWord;
  DetectForm: TDetectForm;
  Hz: Double;
  I, W, Y: Integer;
  RangeLabel: TLabel;
  StartChannel, EndChannel: Byte;

begin
  DeviceIndex := diOnly;

  //If you only have the one board, QueryDeviceInfo() with diOnly will succeed
  //with the right PID. If you have more than one board, or haven't plugged it
  //in, it will fail, and this code will pop up the detector form so the user
  //can pick a board or cancel.
  Status := QueryDeviceInfo(DeviceIndex, @PID, nil, nil, nil, nil);

  {
  PID := $8159;
  Status := ERROR_SUCCESS;
  }

  if (Status <> ERROR_SUCCESS) or not (((PID >= $804A) and (PID <= $805D)) or ((PID >= $814A) and (PID <= $815D))) then begin
    DetectForm := TDetectForm.Create(Self);
    if DetectForm.ShowModal = mrOK then begin
      DeviceIndex := DetectForm.DeviceIndex;
      DetectForm.Free;
      QueryDeviceInfo(DeviceIndex, @PID, nil, nil, nil, nil);
    end
    else begin
      DetectForm.Free;
      Application.Terminate;
      Exit;
    end;
  end;

  case PID of
    $804A..$804E, $814A..$814E: MAXChannels := 32;
    $804F..$8053, $814F..$8153: MAXChannels := 64;
    $8054..$8058, $8154..$8158: MAXChannels := 96;
    $8059..$805D, $8159..$815D: MAXChannels := 128;
  end;

  SetLength(Counts, MAXChannels);

  SetLength(VoltLabel, MAXChannels);
  VoltSpacerBevel.Height := 16 * MAXChannels;
  for I := 0 to High(VoltLabel) do begin
    VoltLabel[I] := TLabel.Create(Self);
    with VoltLabel[I] do begin
      AutoSize := False;
      Alignment := taCenter;
      Layout := tlCenter;
      Align := alTop;
      Top := 16 * I;
      Height := 16;
      Caption := 'Ch ' + IntToStr(I);
      Parent := VoltScroll;
    end;
  end;

  RangeSpacerBevel.Height := ((16+24) div 8) * MAXChannels;
  W := RangeScroll.ClientWidth - 8*2;
  for I := 0 to High(RangeCombo) do begin
    Y := I * (16+24);
    
    RangeLabel := TLabel.Create(Self);
    with RangeLabel do begin
      AutoSize := False;
      Layout := tlBottom;
      Alignment := taCenter;
      Caption := 'Analog Range CH' + IntToStr(I*8) + '-' + IntToStr(I*8 + 7);
      Anchors := [akLeft, akTop, akRight];
      SetBounds(8, Y, W, 16);
      Visible := I*8 < MAXChannels;
      Parent := RangeScroll;
    end;
    Inc(Y, 16);

    RangeCombo[I] := TComboBox.Create(Self);
    with RangeCombo[I] do begin
      Left := 8;
      Top := 16;
      Width := 124;
      Height := 21;
      Style := csDropDownList;
      Anchors := [akLeft, akTop, akRight];
      SetBounds(8, Y, W, 24);
      Visible := I*8 < MAXChannels;
      Parent := RangeScroll;
      
      OnChange := RangeComboChange;

      Items.Text := '0 to 10 Volts'#13#10'10 Volts'#13#10'0 to 5 Volts'#13#10'5 Volts'#13#10'0 to 2 Volts'#13#10'2 Volts'#13#10'0 to 1 Volt'#13#10'1 Volts';
      ItemIndex := 7;
    end;
  end;
  RangeCombo[1].Enabled := False;
  RangeCombo[1].ItemIndex := 2;

  if ADC_QueryCal(DeviceIndex) <> ERROR_SUCCESS then begin
    //this board doesn't have calibration
    CalCheck.Checked := False;
    CalCheck.Visible := False;
    CalCombo.Visible := False;
  end;

  //Stop the counter, in case it was running.
  Hz := 0;
  CTR_StartOutputFreq(DeviceIndex, 0, @Hz);



  StartChannel := 0;
  EndChannel := MAXChannels-1;

  ConfigBufSize := SizeOf(Config);
  RangeComboChange(nil); //Set ranges in config.
  Config.CalibMode := 0; //Take actual data, not internal calibration sources.
  Config.TrigMode := $05; //Scan selected channels each counter rising edge.
  Config.StartStopCH := (EndChannel shl 4) or (StartChannel and $F);
  Config.StartStopChEx := (EndChannel and $F0) or (StartChannel shr 4);
  //Config.Oversample := 0; //No oversample.
  Config.Oversample := 14; //+14 oversample.
  ADC_SetConfig(DeviceIndex, @Config, ConfigBufSize);
end;

procedure TMainForm.RangeComboChange(Sender: TObject);
var
  I: Integer;

begin
  //The ranges in the combo box are listed in order, so that the index is equal
  //to the range code.

  //Config.ChannelRange[0] controls channels $00 through $07;
  //Config.ChannelRange[1] controls channels $08 through $0F; etc.
  for I := $0 to $F do
    Config.ChannelRange[I] := RangeCombo[I].ItemIndex
  ;

  if ADPollTime.Enabled then ADC_SetConfig(DeviceIndex, @Config, ConfigBufSize);
end;

procedure TMainForm.GoButtonClick(Sender: TObject);
var
  Status: LongWord;

begin
  ADC_SetConfig(DeviceIndex, @Config, ConfigBufSize);

  if CalCheck.Checked then begin
    Status := ADC_SetCal(DeviceIndex, PAnsiChar(CalCombo.Text));
    if Status <> ERROR_SUCCESS then begin
      Application.MessageBox(PChar('Calibration Error ' + IntToStr(Status)), 'Cal Error', MB_ICONEXCLAMATION);
      Exit;
    end;
  end;

  CalCombo.Enabled := False;

  GoButton.Enabled := False;
  StopButton.Enabled := True;

  ADPollTime.Enabled := True;
end;

procedure TMainForm.StopButtonClick(Sender: TObject);
begin
  ADPollTime.Enabled := False;

  CalCombo.Enabled := True;

  GoButton.Enabled := True;
  StopButton.Enabled := False;
end;

procedure TMainForm.ADPollTimeTimer(Sender: TObject);
var
  Status: LongWord;
  Channel: Integer;
  Tcjc, TemperatureC, mV, mVTCk: Double;
  S: AnsiString;

begin
  for Channel := 0 to High(Counts) do Counts[Channel] := 0;
  Status := ADC_GetScan(DeviceIndex, @Counts[0]);
  if Status <> ERROR_SUCCESS then begin
    StopButton.Click;

    Application.MessageBox(PChar('Error ' + IntToStr(Status) + ' from ADC_GetScan.'), 'ADC_GetScan Error', MB_ICONEXCLAMATION);

    Exit;
  end;

  Tcjc := Temperature.CFromK(LM335.KFromCt(Counts[8]));
  mVTCk := TypeK.mVFromF(FFromC(Tcjc));

  for Channel := 0 to High(Counts) do begin
    case RangeCombo[Channel div 8].ItemIndex of
      0: mV := 1000 * (10 / 100) / 65536 * (Counts[Channel]        );
      1: mV := 1000 * (20 / 100) / 65536 * (Counts[Channel] - $8000);
      2: mV := 1000 * ( 5 / 100) / 65536 * (Counts[Channel]        );
      3: mV := 1000 * (10 / 100) / 65536 * (Counts[Channel] - $8000);
      4: mV := 1000 * ( 2 / 100) / 65536 * (Counts[Channel]        );
      5: mV := 1000 * ( 4 / 100) / 65536 * (Counts[Channel] - $8000);
      6: mV := 1000 * ( 1 / 100) / 65536 * (Counts[Channel]        );
      7: mV := 1000 * ( 2 / 100) / 65536 * (Counts[Channel] - $8000);
      else mV := NAN;
    end;//case

    case Channel of
      $8: S := Format('LM335A CH%d: %4.1f C', [ Channel, Tcjc ]);
      $9..$F: S := Format('CH#%d: %4.1f mV', [ Channel, mV*100 ]);
      else begin
        TemperatureC := CFromF(TypeK.FFrommV(mV + mVTCk)); // the millivolt reading from the thermocouple and the millivolt CJC from the LM335A
        S := Format('TCouple K #%d: %4.1f C', [ Channel, TemperatureC ]);
      end;
    end;

    VoltLabel[Channel].Caption := S;
  end;
end;

end.
